package notice

import (
	"code.gitea.io/gitea/models"
	"code.gitea.io/gitea/modules/log"
	"code.gitea.io/gitea/modules/setting"
	"encoding/json"
	"github.com/patrickmn/go-cache"
	"time"
)

var noticeCache = cache.New(2*time.Minute, 1*time.Minute)

const (
	NOTICE_CACHE_KEY = "notice"
)

type Notice struct {
	Title    string
	Link     string
	Visible  int //0 invisible, 1 visible
	CommitId string
}

var lock int32 = 0

func GetNewestNotice() (*Notice, error) {
	defer func() {
		if err := recover(); err != nil {
			log.Error("recover error", err)
		}
	}()

	var notice *Notice
	var err error
	if setting.CacheOn {
		notice, err = getNewestNoticeFromCacheAndDisk()
	} else {
		notice, err = getNewestNoticeFromDisk()
	}

	if err != nil {
		return nil, err
	}
	return notice, nil
}

func getNoticeTimeout() time.Duration {
	return time.Duration(setting.CacheTimeOutSecond) * time.Second
}

func getNewestNoticeFromDisk() (*Notice, error) {
	log.Debug("Get notice from disk")
	repoFile, err := models.ReadLatestFileInRepo(setting.UserNameOfNoticeRepo, setting.RepoNameOfNoticeRepo, setting.RefNameOfNoticeRepo, setting.TreePathOfNoticeRepo)
	if err != nil {
		log.Error("GetNewestNotice failed, error=%v", err)
		return nil, err
	}
	notice := &Notice{}
	json.Unmarshal(repoFile.Content, notice)
	if notice.Title == "" {
		return nil, err
	}
	notice.CommitId = repoFile.CommitId
	return notice, nil
}

func getNewestNoticeFromCacheAndDisk() (*Notice, error) {
	v, success := noticeCache.Get(NOTICE_CACHE_KEY)
	if success {
		log.Debug("Get notice from cache,value = %v", v)
		if v == nil {
			return nil, nil
		}
		n := v.(*Notice)
		return n, nil
	}

	notice, err := getNewestNoticeFromDisk()
	if err != nil {
		log.Error("GetNewestNotice failed, error=%v", err)
		noticeCache.Set(NOTICE_CACHE_KEY, nil, 30*time.Second)
		return nil, err
	}
	noticeCache.Set(NOTICE_CACHE_KEY, notice, getNoticeTimeout())
	return notice, nil
}
